echo "Utils.sh included"
LC_NUMERIC="en_US.UTF-8"
LANG="en_US"
HPC="hemera"
OWNER="/bigdata/ParticleFlows/Gregory/MicroOrigami"

function OnHPC
{
	#Tell if I am on the hpc
	local n=${#HPC}
	local hname=$(hostname)
	if [[ ${hname:0:n} == ${HPC} ]]
	then
		true
	else
		false
	fi
}

function HPCAvailable()
{
	#Tell if HPC is available through ssh
	local host="$1"
# 	local port=22
# 	if [ "$#" -ne 1 ]
# 	then
# 		host=${HPC}
# 	fi
# 	nc -z "$host" 22 &>/dev/null
# 	if [ $? -eq 0 ]
# 	then
# 		true
# 	else
# 		false
# 	fi
	if ssh -q ${host} exit
	then
		true
	else
		false
	fi
}

function IsInteger()
{
	#Tell if number is integer
	local number=${1}
    if [[ ${number} =~ ^-?[0-9]+$ ]]
	then
		true
	else
		false
	fi
}

function IsFloat()
{
	#Tell if number is float
	local number=${1}
	printf "%f" "${number}" &> /dev/null
	if [[ "$?" -eq 0 ]]
	then
		true
	else
		false
	fi
}

function ToInteger()
{
	#Convert float to integer
	local number=${1}
	if IsInteger ${number} || IsFloat ${number}
	then
		printf "%.0f" "$number"
	fi
}

function ToFloat()
{
	#Convert integer to float
	#ToFloat a b
	#a: integer to convert
	#b: decimal number (optional)
	local number=${1}
	if IsInteger ${number}
	then
		if [[ "$#" -eq 2 ]]
		then
			local decimals=${2}
			printf "%.${decimals}f" "${number}"
		else
			echo "${number}.0"
		fi
	else
		if [[ "$#" -eq 2 ]]
		then
			local decimals=${2}
			printf "%.${decimals}f" "${number}"
		else
			echo "${number}"
		fi
	fi
}

function StrToNumpy()
{
	#Convert "from:to:step" to array
	#Floats are allowed
	local my_string=${1}
	local my_array=($(echo ${my_string} | tr ':' '\n'))
	local my_size=${#my_array[@]}
	local out
	declare -a out=(-1)
	if [[ ${my_size} -eq 1 ]]
	then
		out=(${my_array[0]})
	elif [[ ${my_size} -eq 2 ]]
	then
		local start=${my_array[0]}
		local end=${my_array[1]}
		if IsInteger ${start} && IsInteger ${end}
		then
			local end=${my_array[1]}
			declare -a out=($(seq ${start} ${end}))
		else
			local start=$(ToFloat ${my_array[0]})
			local end=$(ToFloat ${my_array[1]})
			declare -a out=($(seq ${start} ${end}))
		fi
		local last_index=$((${#out[@]}-1))
		if [ "${out[last_index]}" == "${end}" ]
		then
			unset out[$last_index]
		fi
	elif [[ ${my_size} -eq 3 ]]
	then
		local start=${my_array[0]}
		local end=${my_array[1]}
		local step=${my_array[2]}
		if IsInteger ${start} && IsInteger ${end} && IsInteger ${step}
		then
			declare -a out=($(seq ${start} ${step} ${end}))
		else
			local decs=$(echo -n ${start} | awk -F'.' '{print $2}') #decimal part
			local dece=$(echo -n ${end} | awk -F'.' '{print $2}')
			local decp=$(echo -n ${step} | awk -F'.' '{print $2}')
			local nmax=$(echo "${#decs} ${#dece} ${#decp}" | sed 's/ /\n/g' | sort -n | tail -1)
			local start=$(ToFloat ${my_array[0]} ${nmax})
			local end=$(ToFloat ${my_array[1]} ${nmax})
			local step=$(ToFloat ${my_array[2]} ${nmax})
			declare -a out=($(seq ${start} ${step} ${end}))
		fi
		local last_index=$((${#out[@]}-1))
		if [ "${out[last_index]}" == "${end}" ]
		then
			unset out[$last_index]
		fi
	fi
	echo ${out[*]}
}

function TimeGetPattern
{
	local every="";
	local time_pattern="";
	if [ "$#" -eq 1 ]
	then
		every=${1}
	fi
	case "${every}" in
		"2")    time_pattern="[0-9][0-9][0-9][0-9][02468]";;
		"5")    time_pattern="[0-9][0-9][0-9][0-9][05]";;
		"10")   time_pattern="[0-9][0-9][0-9][0-9]0";;
		"20")   time_pattern="[0-9][0-9][0-9][02468]0";;
		"50")   time_pattern="[0-9][0-9][0-9][05]0";;
		"100")  time_pattern="[0-9][0-9][0-9]00";;
		"200")  time_pattern="[0-9][0-9][02468]00";;
		"500")  time_pattern="[0-9][0-9][05]00";;
		"1000") time_pattern="[0-9][0-9]000";;
		     *) time_pattern="[0-9][0-9][0-9][0-9][0-9]";;
	esac
	echo ${time_pattern}
}

function GetLatestTime()
{
	function PrintUsage
	{
		echo "GetLatestTime:"
		echo "  Get latest time as integer from phi_t?????.dat in owner directory"
		echo "  --help    help"
		echo "  --verbose verbose"
		echo "  --hpc     hpc (set to -1 if local machine)"
		echo "  --owner   owner directory"
		echo "  --width   format width including leading zero"
	}
	local verbose_=false
	local hpc_="-1" # local if <0
	local owner_=$(pwd)
	local width_="-1"
	local longs='help,verbose,hpc:,owner:,width:'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case $1 in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--hpc) hpc_=$2; shift 2;;
			--owner) owner_=$2; shift 2;;
			--width) width_=$2; shift 2;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; return;;
		esac
	done
	unset options
	if ${verbose_}
	then
		echo "Get latest time with "
		echo "  hpc = ${hpc_}"
		echo "  owner = ${owner_}"
		echo "  width = ${width_}"
	fi
	if [[ ${hpc_} =~ ^-?[0-9]+$ && $((hpc_)) -le 0 || "$(hostname)" == *\.cluster ]]
	then
		if test -f "${owner_}/phi_t00000.dat"
		then
			local latestTime=$(ls ${owner_}/phi_t?????.dat | tail -n 1 | awk -F '/' '{print $NF}' | awk -vFS='(_t|.dat)' '{print $2}')
			if [[ ${width_} =~ ^[1-9]$ ]]
			then
				printf "%0${width_}d" $(echo "$latestTime" | sed 's/^0*//')
			else
				ToInteger ${latestTime}
			fi
		else
			echo "-1"
		fi
	else
		if ssh ${hpc_} "test -f ${owner_}/phi_t00000.dat"
		then
			local latestTime=$(ssh ${hpc_} "ls ${owner_}/phi_t?????.dat | tail -n 1 | awk -F '/' '{print \$NF}' | awk -vFS='(_t|.dat)' '{print \$2}'")
			if [[ ${width_} =~ ^[1-9]$ ]]
			then
				printf "%0${width_}d" $(echo "$latestTime" | sed 's/^0*//')
			else
				ToInteger ${latestTime}
			fi
		else
			echo "-1"
		fi
	fi
}

function GetXFromFoldersX()
{
	function PrintUsage
	{
		echo "GetXFromFoldersX"
		echo "  Get list X = 1,2,3,4,5,... from directories hpc:/owner/keyX"
		echo "  --help    help"
		echo "  --verbose verbose"
		echo "  --hpc     hpc (set to -1 if local machine)"
		echo "  --owner   owner directory"
		echo "  --key     key"
		echo "  --every   only keep every N"
	}
	#Get X = 3,4,5,... from directories hpc:/owner/keyX
	local verbose_=false
	local hpc_="-1"
	local owner_=$(pwd)
	local key_=""
	local every_="1"
	local longs='help,verbose,hpc:,owner:,key:,every:'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--hpc) hpc_=$2; shift 2;;
			--owner) owner_=$2; shift 2;;
			--key) key_=$2; shift 2;;
			--every) every_=$2; shift 2;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	unset options
	if ${verbose_}
	then
		echo "Update input file with "
		echo "  hpc = ${hpc_}"
		echo "  owner = ${owner_}"
		echo "  key = ${key_}"
		echo "  every = ${every_}"
	fi
	local my_array=("")
	if [[ ${hpc_} =~ ^-?[0-9]+$ && $((hpc_)) -le 0 || "$(hostname)" == *\.cluster ]]
	then
		if [[ -d ${owner_} && -z $(find ${owner_} -maxdepth 0 -empty) ]]
		then
			my_array=($(find ${owner_} -mindepth 1 -maxdepth 1 -type d -name "${key_}[0-9]*" -printf "%P\n" | sort | sed "s/${key_}//"))
		fi
	else
		if ssh ${hpc_} "[[ -d ${owner_} && -z \$(find ${owner_} -maxdepth 0 -empty) ]]"
		then
			my_array=($(ssh ${hpc_} "find ${owner_} -mindepth 1 -maxdepth 1 -type d -name \"${key_}[0-9]*\" -printf \"%P\n\" | sort | sed \"s/${key_}//\""))
		fi
	fi
	if [[ ${every_} -eq 1 ]]
	then
		echo ${my_array[@]}
	else
		local out
		declare -a out=$(for ((i = 0; i<${#my_array[@]}; i+=${every_})); do echo ${my_array[$i]}; done)
		echo ${out[@]}
	fi
}

function UpdateInputFile()
{
	function PrintUsage
	{
		echo "UpdateInputFile"
		echo "  Update value in the input file needed for the petsc simulation"
		echo "  --help    help"
		echo "  --verbose verbose"
		echo "  --in      input file"
		echo "  --key     name of the variable"
		echo "  --val     value stored by the variable"
	}
	local verbose_=false
	local finput_=""
	local key_=""
	local val_=""
	local longs='help,verbose,in:,key:,val:'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--in) finput_=$2; shift 2;;
			--key) key_=$2; shift 2;;
			--val) val_=$2; shift 2;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	unset options
	if ${verbose_}
	then
		echo "Update input file with "
		echo "  in = ${finput_}"
		echo "  key = ${key_}"
		echo "  val = ${val_}"
	fi
	if [[ -f "${finput_}" ]]
	then
		echo "${key_} = ${val_} in ${finput_}"
		sed -i "s/${key_}[[:space:]]\+=.*/${key_} = ${val_};/g" ${finput_}
	fi
}

function Hemera()
{
	function PrintUsage()
	{
		echo "Hemera"
		echo "  Create bash script for job submission on the HPC"
		echo "  --help    help"
		echo "  --verbose verbose"
		echo "  --in      microorigami input file"
		echo "  --out     output executable bash file (.sh)"
		echo "  --jobname job name upon submission"
		echo "  --time    time limit in hours (integer)"
		echo "  --np      number of tasks"
	}
	local verbose_=false
	local finput_="sim.in"
	local fout_="hemera.sh"
	local jobname_="microorigami"
	local time_="6"
	local np_="4"
	local longs='help,verbose,in:,out:,jobname:,time:,np:'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--in) finput_=$2; shift 2;;
			--out) fout_=$2; shift 2;;
			--jobname) jobname_=$2; shift 2;;
			--time) time_=$(printf "%02d" "$2"); shift 2;;
			--np) np_=$2; shift 2;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	unset options
	if ${verbose_}
	then
		echo "Self folding with "
		echo "  in = ${finput_}"
		echo "  out = ${fout_}"
		echo "  jobname = ${jobname_}"
		echo "  time = ${time_}"
		echo "  np = ${np_}"
	fi
	echo '#!/bin/bash' > ${fout_}
	echo "#SBATCH --job-name=${jobname_}" >> ${fout_}
	echo '#SBATCH --partition=defq' >> ${fout_}
	echo "#SBATCH --time=${time_}:00:00" >> ${fout_}
	echo '#SBATCH --nodes=1' >> ${fout_}
	echo "#SBATCH --ntasks=${np_}" >> ${fout_}
	echo '#SBATCH --error=log_error' >> ${fout_}
	echo '#SBATCH --output=log_output' >> ${fout_}
	echo 'module purge' >> ${fout_}
	echo 'module load gcc/7.3.0 openmpi/2.1.2 make/4.3 cmake/3.20.2 python/3.8.0 blas/3.8.0 lapack/3.8.0 boost/1.78.0 git/2.30.1' >> ${fout_}
	echo 'export IGL_NUM_THREADS=1' >> ${fout_}
	echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/libconfig-1.7.3/lib' >> ${fout_}
	echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/gmp-6.2.1/lib' >> ${fout_}
	echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/mpfr-4.2.1/lib' >> ${fout_}
	echo 'cd $SLURM_SUBMIT_DIR' >> ${fout_}
	echo "mpirun -np \$SLURM_NTASKS ${HOME}/MicroOrigami/src/Main --i ${finput_}" >> ${fout_}
	echo "${fout_} created"
}

function Selffolding()
{
	function SelffoldingUsage()
	{
		echo "SelfFolding:"
		echo "  Run self folding simulations"
		echo "  --help    help"
		echo "  --verbose verbose"
		echo "  --theta   contact angle (from:to:step)"
		echo "  --side    side number (from:to:step)"
		echo "  --ec      elastocapillary number (from:to:step)"
		echo "  --in      input file (.in)"
		echo "  --submit  submit job using sbatch"
	}
	#Create folders thetaXX/sideYY/ecZZ
	local verbose_=false
	local thetas_=("")
	local sides_=("3")
	local ecs_=("1")
	local finput_="Input.in"
	local submit_=false
	local longs='help,verbose,theta:,side:,ec:,in:,submit'
	options=$(getopt --options '' --long "$longs" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		SelffoldingUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) SelffoldingUsage; return;;
			--verbose) verbose_=true; shift;;
			--theta) local str=$2; thetas_=($(StrToNumpy ${str})); shift 2;;
			--side) local str=$2; sides_=($(StrToNumpy ${str})); shift 2;;
			--ec) local str=$2; ecs_=($(StrToNumpy ${str})); shift 2;;
			--in) finput_=$2; shift 2;;
			--submit) submit_=true; shift;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; SelffoldingUsage; return;;
		esac
	done
	unset options
	if ${verbose_}
	then
		echo "Self folding with:"
		echo "  theta = ${thetas_[*]}"
		echo "  side = ${sides_[*]}"
		echo "  ec = ${ecs_[*]}"
		echo "  input file = ${finput_}"
		echo "  submit = ${submit_}"
	fi
	local wdir=$(pwd)
	local theta=""
	local side=""
	local ec=""
	for theta in ${thetas_[@]}
	do
		 for side in ${sides_[@]}
		 do
			for ec in ${ecs_[@]}
			do
				local my_dir="${wdir}/ca${theta}/side${side}/ec${ec}"
				if [[ -d ${my_dir} ]]
				then
					echo "${my_dir} already exists. Nothing to be done."
				else
					echo ${my_dir}
					mkdir -p ${my_dir}
					[ -f ${finput_} ] && cp -v ${finput_} ${my_dir}
					Hemera --in ${finput_} --out ${my_dir}/hemera.sh --jobname c${theta}_s${side}_e${ec} --time 6 --np 8
					UpdateInputFile --in ${my_dir}/${finput_} --key contact_angle --val ${theta}
					UpdateInputFile --in ${my_dir}/${finput_} --key sides --val ${side}
					UpdateInputFile --in ${my_dir}/${finput_} --key Ec --val ${ec}
					if ${submit_}
					then
						cd ${my_dir}
						sbatch hemera.sh
						cd ${wdir}
					fi
				fi
			done
		done
	done
}

function ExportData()
{
	function PrintUsage()
	{
		echo "ExportData:"
		echo "  Export binary data (.dat) to either paraview or human readable format (.txt)"
		echo "  --help       help"
		echo "  --verbose    verbose"
		echo "  --owner      owner directory"
		echo "  --every      export every X, with X = 1,2,5,10,20,50,100"
		echo "  --latestTime export data for latest time only"
		echo "  --tpcl       export three-phase contact line using pvpython (takes ages)"
		echo "  --hinge      export hinges using pvpython (takes ages)"
	}
	local verbose_=false
	local owner_="."
	local every_="1"
	local latestTime_=false
	local tpcl_=false
	local hinge_=false
	local longs='help,verbose,owner:,every:,latestTime,tpcl,hinge'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--owner) owner_=$2; shift 2;;
			--every) every_=$2; shift 2;;
			--latestTime) latestTime_=true; shift;;
			--tpcl) tpcl_=true; shift;;
			--hinge) hinge_=true; shift;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	unset options
	if ${verbose_}
	then
		echo "  Export data with:"
		echo "  owner = ${owner_}"
		echo "  every = ${every_}"
		echo "  latestTime = ${latestTime_}"
		echo "  tpcl = ${tpcl_}"
		echo "  hinge = ${hinge_}"
	fi
	if [[ -d ${owner_} && -z $(find ${owner_} -maxdepth 0 -empty) ]]
	then
		if [[ ! -z $(find ${owner_} -maxdepth 1 -type f -name "phi_t?????.dat") ]]
		then
			local t0=$(expr 0 + $(ls ${owner_}/phi_t?????.dat | head -n 1 | awk -F '/' '{print $NF}' | awk -vFS='(_t|.dat)' '{print $2}'))
			local t1=$(expr 0 + $(ls ${owner_}/phi_t?????.dat | tail -n 1 | awk -F '/' '{print $NF}' | awk -vFS='(_t|.dat)' '{print $2}'))
			local times="${t0}:$((t1+1)):${every_}"
			if ${latestTime_}
			then
				times="$((t1))"
			else
				if [[ ${every_} == "1" ]]
				then
					times="$((t0)):$((t1+1))"
				fi
			fi
			local finput=$(find ${owner_} -maxdepth 1 -type f -name "*\.in" -printf "%f\n")
			local sides=$(grep 'sides' "${owner_}/${finput}" | sed 's/[^-0-9]//g')
			local apothem=""
			if [[ -f log_output ]]
			then
				apothem=$(grep 'apothem' log_output | grep -oE '[0-9]+\.[0-9]+')
			fi
			local wdir=$(pwd)
			echo "  Export data in ${owner_} at time(s) t = -1 and t = ${times}"
			echo "  input file = ${finput}"
			echo "  sides = ${sides}"
			echo "  apothem = ${apothem}"
			cd ${owner_}
			local cmd=""
			for timei in "-1" "${times}"
			do
				local redirection="&> /dev/null"
				if ${verbose_}
				then
					redirection=""
				fi
				cmd="ExportFieldToVTS --f phi --t ${timei} --i ${finput} ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
				if [[ ${sides} -ge 3 ]]
				then
					#pyramide
					if [[ ${timei} == "-1" ]]
					then
						cmd="ExportSolid --f base --t ${timei} --i ${finput} --n ${sides} --stl --txt ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
					fi
					for ((s=0; s<${sides}; s++))
					do
						cmd="ExportSolid --f side${s} --t ${timei} --i ${finput} --tp --stl --txt ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
						cmd="ExportHinge --f hinge${s} --t ${timei} ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
						if IsInteger ${apothem}
						then
							cmd="HingeToVTK.py --f hinge${s} --t ${timei} --apothem ${apothem} ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
						else
							echo "  Error: apothem is undefined in HingeToVTK.py"
						fi
					done
				else
					#flat plate
					if [[ ${timei} == "-1" ]]
					then
						cmd="ExportSolid --f base --t ${timei} --i ${finput} --n 4 --stl --txt ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
					fi
				fi
				cmd="ExportPolylines --f tpcl --t ${timei} ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
				local cloudsize
				if [[ ${timei} == "-1" ]]
				then
					cloudsize=$(find . -maxdepth 1 -type f -name "tpcl??.txt" | sort | tail -n 1 | cut -c 7-8)
				else
					cloudsize=$(find . -maxdepth 1 -type f -name "tpcl??_t?????.txt" | sort | tail -n 1 | cut -c 7-8)
				fi
				echo "  cloud size = ${cloudsize}"
				if IsInteger ${cloudsize}
				then
					((cloudsize++))
				else
					cloudsize="-1"
				fi
				echo "  corrected cloud size = ${cloudsize}"
				cmd="PolylineToVTK.py --f tpcl --t ${timei} --cloudsize ${cloudsize} --writeempty no --info yes ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
				if ${tpcl_}
				then
					find . -type f -name "tpcl??.py" -o -name "tpcl??_t?????.py" -exec bash -c 'cmd="pvpython {} ${redirection}"; echo "  #${cmd}"; eval ${cmd}; unset cmd' \;
				fi
				if ${hinge_}
				then
					find . -type f -name "hinge?.py" -o -name "hinge?_t?????.py" -exec bash -c 'cmd="pvpython {} ${redirection}"; echo "  #${cmd}"; eval ${cmd}; unset cmd' \;
				fi
			done
			cmd="DropToVTK.py --owner ${owner_} ${redirection}"; echo "  #${cmd}"; eval "${cmd}"
			cd ${wdir}
		fi
	fi
}

function ExportScript
{
	function PrintUsage()
	{
		echo "ExportScript (to be called when Selffolding is used before)"
		echo "  Export data on hpc"
		echo "  --verbose verbose"
		echo "  --out     output file name (export.sh)"
		echo "  --time    job time in hours"
		echo "  --submit  submit job using sbatch"
	}
	local verbose_=false
	local fout_="export.sh"
	local time_="4"
	local submit_=false
	local longs='help,verbose,out:,submit'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--out) fout_=$2; shift 2;;
			--time) time_=$2; shift 2;;
			--submit) submit_=true; shift;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
			esac
	done
	unset options
	if ${verbose_}
	then
		echo "  Make export script with:"
		echo "  out = ${fout_}"
		echo "  time = ${time_}"
		echo "  submit = ${submit_}"
	fi
	local thetas=($(GetXFromFoldersX --hpc "-1" --owner "." --key "ca"))
	local sides=($(GetXFromFoldersX --hpc "-1" --owner "./ca${thetas[0]}" --key "side"))
	local ecs=($(GetXFromFoldersX --hpc "-1" --owner "./ca${thetas[0]}/side${sides[0]}" --key "ec"))
	echo "theta = ${thetas[@]}"
	echo "side  = ${sides[@]}"
	echo "ec    = ${ecs[@]}"
	for theta in ${thetas[@]}
	do
		for side in ${sides[@]}
		do
			for ec in ${ecs[@]}
			do
				local owner=$(pwd)
				local wdir="ca${theta}/side${side}/ec${ec}"
				if [[ -d ${wdir} ]]
				then
					cd ${wdir}
					local job="ca${theta}_s${side}_e${ec}"
					echo '#!/bin/bash' > ${fout_}
					echo "#SBATCH --job-name=${job}" >> ${fout_}
					echo '#SBATCH --partition=defq' >> ${fout_}
					echo "#SBATCH --time=${time_}:00:00" >> ${fout_}
					echo '#SBATCH --nodes=1' >> ${fout_}
					echo "#SBATCH --ntasks=1" >> ${fout_}
					echo '#SBATCH --error=log_export_error' >> ${fout_}
					echo '#SBATCH --output=log_export_output' >> ${fout_}
					echo 'module purge' >> ${fout_}
					echo 'module load gcc/7.3.0 openmpi/2.1.2 make/4.3 cmake/3.20.2 python blas lapack boost git' >> ${fout_}
					echo 'export IGL_NUM_THREADS=1' >> ${fout_}
					echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/libconfig-1.7.3/lib' >> ${fout_}
					echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/gmp-6.2.1/lib' >> ${fout_}
					echo 'export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/local/mpfr-4.2.1/lib' >> ${fout_}
					echo 'export PATH=$PATH:$HOME/Paraview/ParaView-5.9.1-MPI-Linux-Python3.8-64bit/bin' >> ${fout_}
					echo 'export PATH=$PATH:$HOME/MicroOrigami/scripts' >> ${fout_}
					echo 'source Utils.sh' >> ${fout_}
					echo 'cd $SLURM_SUBMIT_DIR' >> ${fout_}
					echo 'rm -f *txt *stl *vts *vtk *py' >> ${fout_}
					#echo "ExportData --tpcl --every 1 --verbose" >> ${fout_} #Takes ages
					echo "ExportData --every 1 --verbose" >> ${fout_}
					echo "${fout_} created"
					if ${submit_}
					then
						sbatch ${fout_}
					fi
					cd ${owner}
				else
					echo "  ${wdir} does not exist"
				fi
			done
		done
	done
}

function RemoveRawData()
{
	function PrintUsage
	{
		echo "Remove raw data from owner directory"
		echo "  --help    help"
		echo "  --verbose verbose"
		echo "  --owner   owner directory"
	}
	local verbose_=false
	local owner_="."
	local longs='help,verbose,owner:'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--owner) owner_=$2; shift 2;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	unset options
	if ${verbose_}
	then
		echo "  Remove data with:"
		echo "  owner = ${owner_}"
	fi
	if [[ -d ${owner_} ]]
	then
		echo "  Remove raw data from ${owner_}"
		local redirection="&> /dev/null"
		if ${verbose_}
		then
			redirection=""
		fi
		eval "rm -fv ${owner_}/*\.dat ${owner_}*\.dat.info ${redirection}"
	fi
}

function CopyRawData()
{
	function PrintUsage()
	{
		echo "CopyRawData"
		echo "  Copy raw data from server to local directory"
		echo "  --help       help"
		echo "  --verbose    verbose"
		echo "  --hpc        hpc"
		echo "  --owner      owner directory on hpc"
		echo "  --every      copy every X, with X = 1,2,5,10,20,50,100"
		echo "  --latestTime copy data at latest time only"
		echo "  --to         specific directory, where to copy the data"
	}
	local verbose_=false
	local hpc_="${HPC}"
	local owner_="$HOME"
	local every_="1"
	local latestTime_=false
	local to_=""
	local longs='help,verbose,hpc:,owner:,every:,latestTime,to:'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--hpc) hpc_=$2; shift 2;;
			--owner) owner_=$2; shift 2;;
			--every) every_=$2; shift 2;;
			--latestTime) latestTime_=true; shift;;
			--to) to_=$2; shift 2;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	unset options
	if [[ -z ${to_} ]]
	then
		to_=$(pwd)/$(basename ${owner_})
	fi
	if ${verbose_}
	then
		echo "  Copy raw data with:"
		echo "  hpc = ${hpc_}"
		echo "  owner = ${owner_}"
		echo "  every = ${every_}"
		echo "  latestTime = ${latestTime_}"
		echo "  to = ${to_}"
	fi
	if [[ "${hpc_}" =~ ^-?[0-9]+$ && $((hpc_)) -le 0 ]]
	then
		echo "  Provide ssh server (hpc = -1)"
		return
	fi
	if OnHPC ${hpc_}
	then
		echo "  Copy function on ${hpc_} ignored"
		return
	fi
	if ! HPCAvailable ${hpc_}
	then
		echo "${hpc_} is NOT available"
		return
	fi
	local time_pattern=$(TimeGetPattern ${every_})
	if ${latestTime_}
	then
		time_pattern=$(GetLatestTime --hpc ${hpc_} --owner ${owner_} --width 5)
	fi
	if ${verbose_}
	then
		echo "  time_pattern = ${time_pattern}"
	fi
	echo "  Copy from ${hpc_}:${owner_} to ${to_}"
	local redirection="&> /dev/null"
	if ${verbose_}
	then
		redirection=""
	fi
	mkdir -p ${to_}
	eval "scp ${hpc_}:${owner_}/base.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/phi.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/tpcl.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/gap[0-9].dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/hinge[0-9].dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/side[0-9].dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/log_output ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/log_error ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/*.in ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/*.txt ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/drop_t${time_pattern}.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/phi_t${time_pattern}.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/tpcl_t${time_pattern}.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/hinge[0-9]_t${time_pattern}.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/side[0-9]_t${time_pattern}.dat ${to_} ${redirection}"
}

function CopyExportedData()
{
	function PrintUsage()
	{
		echo "CopyExportedData"
		echo "  Copy exported data from server to local directory"
		echo "  --help       help"
		echo "  --verbose    verbose"
		echo "  --hpc        hpc"
		echo "  --owner      owner directory on hpc"
		echo "  --every      copy every X, with X = 1,2,5,10,20,50,100"
		echo "  --latestTime copy data at latest time only"
		echo "  --to         specific directory, where to copy the data"
	}
	local verbose_=false
	local hpc_="${HPC}"
	local owner_="$HOME"
	local every_="1"
	local latestTime_=false
	local to_=""
	local longs='help,verbose,hpc:,owner:,every:,latestTime,to:'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--verbose) verbose_=true; shift;;
			--hpc) hpc_=$2; shift 2;;
			--owner) owner_=$2; shift 2;;
			--every) every_=$2; shift 2;;
			--latestTime) latestTime_=true; shift;;
			--to) to_=$2; shift 2;;
			--) shift; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	unset options
	if [[ -z ${to_} ]]
	then
		to_=$(pwd)/$(basename ${owner_})
	fi
	if ${verbose_}
	then
		echo "  Copy exported data with:"
		echo "  hpc = ${hpc_}"
		echo "  owner = ${owner_}"
		echo "  every = ${every_}"
		echo "  latestTime = ${latestTime_}"
		echo "  to = ${to_}"
	fi
	if [[ "${hpc_}" =~ ^-?[0-9]+$ && $((hpc_)) -le 0 ]]
	then
		echo "  Provide ssh server (hpc = -1)"
		return
	fi
	if OnHPC ${hpc_}
	then
		echo "  Copy function on ${hpc_} ignored"
		return
	fi
	if ! HPCAvailable ${hpc_}
	then
		echo "  ${hpc_} is NOT available"
		return
	fi
	local time_pattern=$(TimeGetPattern ${every_})
	if ${latestTime_}
	then
		time_pattern=$(GetLatestTime --hpc ${hpc_} --owner ${owner_} --width 5)
	fi
	if ${verbose_}
	then
		echo "  time_pattern = ${time_pattern}"
	fi
	echo "  Copy from ${hpc_}:${owner_} to ${to_}"
	local redirection="&> /dev/null"
	if ${verbose_}
	then
		redirection=""
	fi
	mkdir -p ${to_}
	eval "scp ${hpc_}:${owner_}/force.dat ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/base.txt ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/hinge0_t${time_pattern}.txt ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/side0_t${time_pattern}.txt ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/tpcl??_t${time_pattern}.txt ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/base.stl ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/*_t${time_pattern}.vtk ${to_} ${redirection}"
	eval "scp ${hpc_}:${owner_}/*_t${time_pattern}.stl ${to_} ${redirection}"
}
